/* 
 * 
 * Copyright (C) 2009-2011 Colomban Wendling <ban@herbesfolles.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifndef H_CTPL_H
#define H_CTPL_H

#define H_CTPL_H_INSIDE

#include "ctpl-environ.h"
#include "ctpl-eval.h"
#include "ctpl-lexer-expr.h"
#include "ctpl-lexer.h"
#include "ctpl-parser.h"
#include "ctpl-io.h"
#include "ctpl-input-stream.h"
#include "ctpl-output-stream.h"
#include "ctpl-token.h"
#include "ctpl-value.h"
#include "ctpl-version.h"

#undef H_CTPL_H_INSIDE

#endif /* guard */
